--------------------------------------------------------
-- Archivo creado  - viernes-diciembre-27-2013   
--------------------------------------------------------
--------------------------------------------------------
--  DDL for Package PKG_PARLAMENTARI
--------------------------------------------------------

spool C:\tfc\PKG_PARLAMENTARI.log

  CREATE OR REPLACE PACKAGE "PKG_PARLAMENTARI" 
as

  ----------------------------
  --PACKAGE ABM PARLAMENTARI--
  ----------------------------
--Procediment per donar d'alta un parlamentari.
procedure prc_alta_parlamentari(
    p_nomcambra       varchar2,
    p_nomcognoms      varchar2,
    p_partitpolitic   varchar2,
    p_idestaactiu     varchar2
    );

  --Procediment per donar de baixa un parlamentari.
procedure prc_baixa_parlamentari(
    p_numparlamentari number
    );

  --Procediment per donar modificar les dades d'un parlamentari.
procedure prc_modifica_parlamentari(
    p_numparlamentari number,
    p_nomcambra       varchar2,
    p_nomcognoms      varchar2,
    p_partitpolitic   varchar2,
    p_idestaactiu     varchar2
    );

end pkg_parlamentari;

/
--------------------------------------------------------
--  DDL for Package Body PKG_PARLAMENTARI
--------------------------------------------------------

  CREATE OR REPLACE PACKAGE BODY "TFCUSER"."PKG_PARLAMENTARI" 
as

 -------------------------
 --PRC_ALTA_PARLAMENTARI--
 -------------------------
--Procediment per donar d'alta un parlamentaris
procedure prc_alta_parlamentari(
          p_nomcambra        varchar2,
          p_nomcognoms       varchar2,
          p_partitpolitic    varchar2,
          p_idestaactiu        varchar2
      )as

      --variable per guardar el valor d'un identificador nic amb que es
      --registrar el procediment a la taula de LOGPROCEDIMENTS
      v_idproc number;
      v_numparlamentari number;

      --variable on es guarden els valors del nou parlamentari
      v_parl parlamentari%rowtype;

      --Guarda els parmetres d'entrada a la variable v_parl
      begin

      v_parl.numparlamentari := seq_parlamentaris.NEXTVAL;
      v_parl.nomcambra := p_nomcambra;
      v_parl.nomcognoms := p_nomcognoms;
      v_parl.partitpolitic := p_partitpolitic;
      v_parl.idestaactiu  := p_idestaactiu;

      insert into parlamentari values v_parl;

      commit;

      --Instruccions per registrar el procediment a la taula de LOGPROCEDIMENTS
      v_idproc := seq_log.nextval;
      v_numparlamentari := v_parl.numparlamentari;
      pkg_log.reg_log(v_idproc,'PRC_ALTA_PARLAMENTARI - PARAMETRES: '||v_numparlamentari||', '||p_nomcambra||', '||p_nomcognoms||', '||p_partitpolitic||', '||p_idestaactiu||'','NO HI HA PARAMETRES DE SORTIDA');

    --tractament d'excepcions
    exception
	  -- Si l'execuci del procediment provoca una excepci es registra a la taula LOGPROCEDIMENT seguit
    -- del text "ERROR: " + detall de l'error.
       when others then
       v_idproc := seq_log.nextval;
	   pkg_log.reg_log(v_idproc, 'ERROR', substr(sqlerrm,1,500));

end prc_alta_parlamentari;


 -------------------------
 --PRC_BAIXA_PARLAMENTARI--
 -------------------------
--Procediment per donar de baixa un parlamentari
procedure prc_baixa_parlamentari(
          p_numparlamentari  number
      )as

      --variable per guardar el valor d'un identificador nic amb que es
      --registrar el procediment a la taula de LOGPROCEDIMENTS
      v_idproc number;
      --variable on es guarden els valors del nou parlamentari
       cursor c_parl is
          select * from parlamentari
           where  numparlamentari = p_numparlamentari
           for update of numparlamentari;

      --variable on anirem guardant el que passa pel cursor per poder tractar les dades
       i c_parl%rowtype;

      begin

      --Instruccions per registrar el procediment a la taula de LOGPROCEDIMENTS
      v_idproc := seq_log.nextval;
      pkg_log.reg_log(v_idproc,'PRC_BAIXA_PARLAMENTARI - PARAMETRE: '||p_numparlamentari||'','NO HI HA PARAMETRES DE SORTIDA');

      open c_parl;
            fetch c_parl into i;
            if c_parl%found then
              close c_parl;
              -- Elimina el registre de la taula PARLAMENTARI que t el nombre de parlamentari que s'ha passat per parmetre.
              if i.idestaactiu = 'A' then
                update parlamentari set idestaactiu = 'B' where numparlamentari = p_numparlamentari;
                commit;
              else
                dbms_output.put_line ('ERROR: EL PARLAMENTARI QUE HS INTRODUT ('||p_numparlamentari ||') JA ESTA DONAT DE BAIXA');
              end if;

            else
               dbms_output.put_line ('ERROR: EL PARLAMENTARI QUE HAS INTRODUIT ('||p_numparlamentari||') NO EXISTEIX A LA BASE DE DADES.');
            end if;
      commit;

      --tractament d'excepcions
      exception
	  -- Si l'execuci del procediment provoca una excepci es registra a la taula LOGPROCEDIMENT seguit
    -- del text "ERROR: " + detall de l'error.
       when others then
       v_idproc := seq_log.nextval;
	   pkg_log.reg_log(v_idproc, 'ERROR', substr(sqlerrm,1,500));

end prc_baixa_parlamentari;

 -----------------------------
 --PRC_MODIFICA_PARLAMENTARI--
 -----------------------------
--Procediment per modificar un parlamentari
procedure prc_modifica_parlamentari(
      p_numparlamentari  number,
      p_nomcambra        varchar2,
      p_nomcognoms       varchar2,
      p_partitpolitic    varchar2,
      p_idestaactiu        varchar2
    )as

      --variable per guardar el valor d'un identificador nic amb que es
      --registrar el procediment a la taula de LOGPROCEDIMENTS
      v_idproc number;
      --variable on es guarden els valors per modificar del parlamentari amb numParlamentari = p_numparlamentari
      v_parl parlamentari%rowtype;

      --cursor que retorna el registre de la taula de PARLAMENTARI del parlamentari amb el numParlamentari = p_numparlamentari
      cursor c_parl is
          select * from parlamentari
           where numparlamentari = p_numparlamentari
           for update of numparlamentari;

      --variable on es guarden les dades obtingudes en el cursor c_parl per poder
      --tractarles posteriorment
      i c_parl%rowtype;


      begin

      --Instruccions per registrar el procediment a la taula de LOGPROCEDIMENTS
      v_idproc := seq_log.nextval;
      pkg_log.reg_log(v_idproc,'PRC_MODIFICA_PARLAMENTARI - PARAMETRES: '||p_numparlamentari||', '||p_nomcambra||', '||p_nomcognoms||', '||p_partitpolitic||', '||p_idestaactiu||'','NO HI HA PARAMETRES DE SORTIDA');

      open c_parl;
              fetch c_parl into i;
              if c_parl%found then
                close c_parl;

                for i in c_parl loop
                  -- Els valors que s'han deixat en blanc al cridar el procediment es deixen com estan a la taula PARLAMENTARI.
                  v_parl.numparlamentari := p_numparlamentari;
                  v_parl.nomcambra       := nvl(p_nomcambra,i.nomcambra);
                  v_parl.nomcognoms      := nvl(p_nomcognoms,i.nomcognoms);
                  v_parl.partitpolitic   := nvl(p_partitpolitic,i.partitpolitic);
                  v_parl.idestaactiu     := nvl(p_idestaactiu, i.idestaactiu);

                  update parlamentari set row = v_parl where current of c_parl;
                end loop;
              else
                dbms_output.put_line ('ERROR: EL PARLAMENTARI QUE HAS INTRODUIT ('||p_numparlamentari||') NO EXISTEIX A LA BASE DE DADES.');
              end if;
              commit;

    --tractament d'excepcions
    exception
	  -- Si l'execuci del procediment provoca una excepci es registra a la taula LOGPROCEDIMENT seguit
    -- del text "ERROR: " + detall de l'error.
       when others then
       v_idproc := seq_log.nextval;
	   pkg_log.reg_log(v_idproc, 'ERROR', substr(sqlerrm,1,500));

end prc_modifica_parlamentari;

end pkg_parlamentari;

/

spool off;