--------------------------------------------------------
-- Archivo creado  - domingo-diciembre-29-2013   
--------------------------------------------------------
--------------------------------------------------------
--  DDL for Package PKG_CONSULTES
--------------------------------------------------------

spool C:\tfc\PKG_CONSULTES.log

  CREATE OR REPLACE PACKAGE "PKG_CONSULTES" as

  ---------------------
  --PACKAGE CONSULTES--
  ---------------------

  --------------------------------
  --C1_DESPESES_PARLAMENTARI_ANY--
  --------------------------------
  --procediment per fer un llistat de les despeses associades a un parlamentari
  --en l'any indicat
  procedure c1_despeses_parlamentari_any(
        p_numparlamentari number,
        p_nomcambra       varchar2,
        p_any             number
  );


  ------------------------------
  --C2_LLISTAT_PARL_CAMBRA_ANY--
  ------------------------------
  --procediment per fer una llista dels parlamentaris de la cambra entrada per
  --parmetre amb les seves despeses associades
  procedure c2_llistat_parl_cambra_any(
        p_nomcambra varchar2,
        p_any       number
  );


  ------------------------
  --C3_DESPESA_TOTAL_ANY--
  ------------------------
 --procediment per fer un llistat de les despeses de tots els parlaments en l'any
 --indicat pel parmetre. Les despeses es desglossen segons si sn de les cambres
 --o els parlamentaris
 procedure c3_despesa_total_any(
        p_any   number
 );


  --------------------------------
  --C4_DESPESES_CATEGORIA_CAMBRA--
  --------------------------------
 --procediment per fer un llistat de les despeses d'una cambra i any indicats pels
 --parmetres d'entrada desglossades segons la categoria de les despeses
 procedure c4_despeses_categoria_cambra(
        p_nomcambra varchar2,
        p_any       number
 );


  -------------------------
  --C5_PARL_SUPEREN_MITJA--
  -------------------------
 --procediment per fer un llistat dels parlamentaris de la cambra indicada que han
 --superat la mitjana de les despeses de tots els parlamentaris
 procedure c5_parl_superen_mitja(
        p_nomcambra varchar2,
        p_any       number
 );


  ----------------------
  --C6_ESTAT_COMPTABLE--
  ----------------------
 --procediment per calcular l'estat comptable de totes les cambres en l'any
 --indicat pel parmetre d'entrada
 procedure c6_estat_comptable(
        p_any     number
 );

end pkg_consultes;

/
--------------------------------------------------------
--  DDL for Package Body PKG_CONSULTES
--------------------------------------------------------

  CREATE OR REPLACE PACKAGE BODY "TFCUSER"."PKG_CONSULTES" as

  --------------------------------
  --C1_DESPESES_PARLAMENTARI_ANY--
  --------------------------------
  --procediment per fer un llistat de les despeses associades a un parlamentari
  --en l'any indicat
  procedure c1_despeses_parlamentari_any(
        p_numparlamentari number,
        p_nomcambra       varchar2,
        p_any             number
  ) as

      --variable per guardar el valor d'un identificador nic amb que es
      --registrar el procediment a la taula de LOGPROCEDIMENTS
      v_idproc number;
      
      --variables per saber si la cambra i l'any indicats es troben a la bd
      trobat boolean;
      trobatcam boolean;
    
     --cursor per saber si l'any existeix a la bbdd
      cursor c_any is
        select distinct anydespesa
        from despesa;
        
     --variable on guardar l'any
     anydes c_any%rowtype;
     
      --cursor per saber si l'any existeix a la bbdd
      cursor c_cam is
        select distinct nomcambra
        from despesa;
        
     --variable on guardar l'any
     cam c_cam%rowtype;
     
    --cursor per assignar el nom del parlamentari
    cursor c_parl is
      select nomcognoms
        from parlamentari
        where numparlamentari = p_numparlamentari;
    
    --guardem el nom buscat amb el cursor anterior a la variable 
    parl c_parl%rowtype;

   --cursor que ens retornar totes les despeses asociades al parlamentari i
   --l'any indicats en els parmetres, ordenades de ms a menys import.
      cursor c_desparl is
        select distinct pl.numparlamentari, pl.nomcognoms, pl.partitpolitic, pl.nomcambra, d.iddespesa, d.idcategoria, d.justificaciodespesa, d.import, d.anydespesa
          from parlamentari pl, despesa d
          where pl.numparlamentari = p_numparlamentari
          and pl.numparlamentari = d.numparlamentari
          and pl.nomcambra = p_nomcambra
          and d.anydespesa = p_any
          order by d.import desc;

      --variable on es guarden els valors obtinguts abans
      i c_desparl%rowtype;

  begin

      --Instruccions per registrar el procediment a la taula de LOGPROCEDIMENTS
      v_idproc := seq_log.nextval;
      pkg_log.reg_log(v_idproc,'C1_DESPESES_PARLAMENTARI_ANY - PARAMETRES: '||p_numparlamentari||', '||p_nomcambra||', '||p_any||'', 'LLISTAT DE SORTIDA PER STDOUT');

      -- Augmentem el tamany del buffer de l'output suficient perqu cpiga tot el missatge.
      dbms_output.enable(1000000);
      
      --guardem el nom del parlamentari
       open c_parl;
        fetch c_parl into parl;

        if c_parl%found then
        close c_parl;
        end if;

      dbms_output.put_line('----------------------------------------------');
      dbms_output.put_line('LLISTAT DE DESPESES DE: '||parl.nomcognoms);
      dbms_output.put_line('----------------------------------------------');
      dbms_output.new_line;
      
      --inicialitzem les variables
      trobat := false;
      trobatcam := false;
      
        -- S'obre el cursor amb les dades dels anys
     open c_any;
        fetch c_any into anydes;

        if c_any%found then
        close c_any;
        
        --mirem si l'any indicat es troba a la bd
        for anydes in c_any loop
          if anydes.anydespesa = p_any then
          trobat := true;
          end if;
          end loop;
          end if;
          if trobat = false then 
            dbms_output.put_line('LA CONSULTA NO HA TROBAT L''ANY INDICAT');
           
          else
          
            -- S'obre el cursor amb les dades de les cambres
          open c_cam;
        fetch c_cam into cam;

        if c_cam%found then
        close c_cam;
        
        --mirem si l'any indicat es troba a la bd
        for cam in c_cam loop
          if cam.nomcambra = p_nomcambra then
          trobatcam := true;
            end if;
          end loop;
          end if;
          if trobatcam = false then 
            dbms_output.put_line('LA CONSULTA NO HA TROBAT LA CAMBRA INDICADA');
           
          else

      -- S'obre el cursor amb les dades de les despeses del parlamentari indicat
      open c_desparl;
        fetch c_desparl into i;

        if c_desparl%found then
        close c_desparl;
        for i in c_desparl loop
          -- Si el cursor retorna registres s'escriuen al buffer de l'output.
          dbms_output.put_line('NOMBRE DE PARLAMENTARI:  '||i.numparlamentari);
		    	dbms_output.put_line('NOM I COGNOMS:           '||i.nomcognoms);
          dbms_output.put_line('PARTIT POLITIC:          '||i.partitpolitic);
          dbms_output.put_line('NOM CAMBRA:              '||i.nomcambra);
          dbms_output.put_line('DESPESA:                 '||i.iddespesa);
          dbms_output.put_line('CATEGORIA DESPESA:       '||i.idcategoria);
          dbms_output.put_line('JUSTIFICACIO DESPESA:    '||i.justificaciodespesa);
          dbms_output.put_line('IMPORT DESPESA:          '||i.import);
          dbms_output.put_line('ANY DESPESA:             '||i.anydespesa);
          dbms_output.put_line('----');
          dbms_output.new_line;
        end loop;
      else
        dbms_output.put_line('----------------------------------------------------------------------');
        dbms_output.put_line('LA CONSULTA NO HA TROBAT CAP PARLAMENTARI AMB DESPESES L''ANY ' ||p_any);
        dbms_output.put_line('-----------------------------------------------------------------------');
  		  dbms_output.new_line;
		  end if;
    end if;
    end if;

    --tractament d'excepcions
       exception
	  -- Si l'execuci del procediment provoca una excepci es registra a la taula LOGPROCEDIMENT seguit
    -- del text "ERROR: " + detall de l'error.
       when others then
       v_idproc := seq_log.nextval;
	   pkg_log.reg_log(v_idproc, 'ERROR', substr(sqlerrm,1,500));
     dbms_output.put_line('No s''ha trobat algun dels parmetres d''entrada: '||p_numparlamentari||', ' ||p_nomcambra|| 'o ' ||p_any);

  end c1_despeses_parlamentari_any;


  ------------------------------
  --C2_LLISTAT_PARL_CAMBRA_ANY--
  ------------------------------
 --procediment per fer una llista dels parlamentaris de la cambra entrada per
 --parmetre amb les seves despeses associades
  procedure c2_llistat_parl_cambra_any(
        p_nomcambra varchar2,
        p_any       number
  ) as

      --variable per guardar el valor d'un identificador nic amb que es
      --registrar el procediment a la taula de LOGPROCEDIMENTS
      v_idproc number;
      
      --variable per emmagatzermar el nom de la cambra on fem el llistat de
      --parlamentaris i les seves despeses totals
      v_nom_cam varchar2 (150);
      
      --variable
      trobat boolean;
      
       --cursor per saber si l'any existeix a la bbdd
      cursor c_any is
        select distinct anydespesa
        from despesa;
        
     --variable on guardar l'any
     anydes c_any%rowtype;
 
       --cursor que ens retornar la suma de les despeses asociades als
       --parlamentaris de la cambra entrada com a parmetre l'any indicat
       --ordentats de ms a menys despesa total.
      cursor c_llistat_parl is
      select  pl.nomcognoms, pl.partitpolitic, sum(d.import) as suma_despeses
        from despesa d, parlamentari pl
          where pl.numparlamentari = d.numparlamentari
          and pl.nomcambra = p_nomcambra
          and d.anydespesa = p_any
          group by pl.nomcognoms, pl.partitpolitic
          order by sum(d.import) desc;

      --variable on es guarden els valors obtinguts abans
      i c_llistat_parl%rowtype;

    begin

     --Instruccions per registrar el procediment a la taula de LOGPROCEDIMENTS
      v_idproc := seq_log.nextval;
      pkg_log.reg_log(v_idproc,'C2_LLISTAT_PARL_CAMBRA_ANY - PARAMETRES: '||p_nomcambra||', '||p_any||'', 'LLISTAT DE SORTIDA PER STDOUT');

      -- Augmentem el tamany del buffer de l'output suficient perqu cpiga tot el missatge.
      dbms_output.enable(1000000);

      dbms_output.put_line('--------------------------------------------');
      dbms_output.put_line('LLISTAT DE DESPESES DELS PARLAMENTARIS');
      dbms_output.put_line('DE LA CAMBRA: '||p_nomcambra);
      dbms_output.put_line('EN L''ANY: '||p_any);
      dbms_output.put_line('--------------------------------------------');
      dbms_output.new_line;
       
       --inicialitzaci de la variable trobat
       trobat := false;
    
    
      -- S'obre el cursor amb les dades dels anys
     open c_any;
        fetch c_any into anydes;

        if c_any%found then
        close c_any;
        
        --mirem si l'any indicat es troba a la bd
        for anydes in c_any loop
          if anydes.anydespesa = p_any then
          trobat := true;
          end if;
          end loop;
          end if;
          if trobat = false then 
            dbms_output.put_line('LA CONSULTA NO HA TROBAT L''ANY INDICAT');
           
    else
      -- S'obre el cursor amb les dades de les despeses del parlamentari indicat
      open c_llistat_parl;
        fetch c_llistat_parl into i;

        if c_llistat_parl%found then
        close c_llistat_parl;
        for i in c_llistat_parl loop
          -- Si el cursor retorna registres s'escriuen al buffer de l'output.
         	dbms_output.put_line('NOM I COGNOMS:  '||i.nomcognoms);
          dbms_output.put_line('PARTIT POLITIC: '||i.partitpolitic);
          dbms_output.put_line('DESPESA TOTAL : '||to_char(i.suma_despeses));
          dbms_output.put_line('---');
          dbms_output.new_line;
        end loop;
      else
        dbms_output.put_line('-----------------------------------------------------');
        dbms_output.put_line('LA CONSULTA NO HA TROBAT CAP CAMBRA EN L''ANY INDICAT');
        dbms_output.put_line('-----------------------------------------------------');
  		  dbms_output.new_line;
		  end if;
    end if;
    
    --tractament d'excepcions
    exception
	  -- Si l'execuci del procediment provoca una excepci es registra a la taula LOGPROCEDIMENT seguit
    -- del text "ERROR: " + detall de l'error.
       when others then
       v_idproc := seq_log.nextval;
	   pkg_log.reg_log(v_idproc, 'ERROR', substr(sqlerrm,1,500));
     dbms_output.put_line('No s''ha trobat algun dels parmetres d''entrada: ' ||p_nomcambra|| 'o ' ||p_any);

  end c2_llistat_parl_cambra_any;


  ------------------------
  --C3_DESPESA_TOTAL_ANY--
  ------------------------
 --procediment per fer un llistat de les despeses de tots els parlaments en l'any
 --indicat pel parmetre. Les despeses es desglossen segons si sn de les cambres
 --o els parlamentaris
 procedure c3_despesa_total_any(
        p_any   number
 ) as

      --variable per guardar el valor d'un identificador nic amb que es
      --registrar el procediment a la taula de LOGPROCEDIMENTS
      v_idproc number;

      --variables on emmagatzemarem els diferents valors calculats que ens
      --demanen
      avg_des_parl  number(8,2);
      sum_des_parl  number;
      sum_des_cam   number;
      sum_des_tot   number;
      trobat boolean;
      
      --cursor per saber si l'any existeix a la bbdd
      cursor c_any is
        select distinct anydespesa
        from despesa;
        
     --variable on guardar l'any
     anydes c_any%rowtype;
        

      --cursor que ens selecciona els noms de les diferents cambres
      cursor c_nomcam is
          select distinct nomcambra, numparlamentaris
          from cambra;

      --variable on guardar els resultats del cursor anterior
      i c_nomcam%rowtype;

 begin

      --Instruccions per registrar el procediment a la taula de LOGPROCEDIMENTS
      v_idproc := seq_log.nextval;
      pkg_log.reg_log(v_idproc,'C3_DESPESA_TOTAL_ANY - PARAMETRE: '||p_any||'', 'LLISTAT DE SORTIDA PER STDOUT');

      -- Augmentem el tamany del buffer de l'output suficient perqu cpiga tot el missatge.
      dbms_output.enable(1000000);

      --Capalera output
      dbms_output.put_line('--------------------------------------------');
      dbms_output.put_line('LLISTAT DE DESPESES DE LES CAMBRES');
      dbms_output.put_line('EN L''ANY: '||p_any);
      dbms_output.put_line('--------------------------------------------');
      dbms_output.new_line;

      --inicialitzem les variables
      avg_des_parl := '0';
      sum_des_parl := '0';
      sum_des_cam  := '0';
      sum_des_tot  := '0';
      trobat := false;
    
    
      -- S'obre el cursor amb les dades dels anys
     open c_any;
        fetch c_any into anydes;

        if c_any%found then
        close c_any;
        
        --mirem si l'any indicat es troba a la bd
        for anydes in c_any loop
          if anydes.anydespesa = p_any then
          trobat := true;
          end if;
          end loop;
          if trobat = false then 
            dbms_output.put_line('LA CONSULTA NO HA TROBAT L''ANY INDICAT');
           
          else
        
       -- S'obre el cursor amb les dades de les cambres
     open c_nomcam;
        fetch c_nomcam into i;

        if c_nomcam%found then
        close c_nomcam;

        --fem els clculs desitjats per cada cambra trobada
      for i in c_nomcam loop
          select avg(d.import) into avg_des_parl
                from despesa d, parlamentari pl
                where i.nomcambra = pl.nomcambra
                and pl.numparlamentari = d.numparlamentari
                and d.anydespesa = p_any;
                if avg_des_parl is null
                then avg_des_parl := '0';
                end if;
          select sum(d.import) into sum_des_parl
                from despesa d, parlamentari pl
                where i.nomcambra = pl.nomcambra
                and pl.numparlamentari = d.numparlamentari
                and d.anydespesa = p_any;
                if sum_des_parl is null
                then sum_des_parl := '0';
                end if;
           select sum(d.import) into sum_des_cam
                from despesa d
                where i.nomcambra = d.nomcambra
                and d.anydespesa = p_any
                and d.idtipusdespesa = 'AC';
                if sum_des_cam is null
                then sum_des_cam := '0';
                end if;
          sum_des_tot := sum_des_parl + sum_des_cam;

           -- Presentem els resultats per l'stdout.
         	dbms_output.put_line('NOM CAMBRA:                       '||i.nomcambra);
          dbms_output.put_line('NOMBRE DE PARLAMENARIS:           '||i.numparlamentaris);
          dbms_output.put_line('DESPESA MITJA DELS PARLAMENTARIS: '||to_char(avg_des_parl));
          dbms_output.put_line('DESPESA TOTAL DELS PARLAMENTARIS: '||to_char(sum_des_parl));
          dbms_output.put_line('DESPESA TOTAL DE LES CAMBRES:     '||to_char(sum_des_cam));
          dbms_output.put_line('DESPESA TOTAL:                    '||to_char(sum_des_tot));
          dbms_output.put_line('---');
          dbms_output.new_line;
        end loop;
     
		  end if;
     
     end if;
    end if;
     --tractament d'excepcions
     exception
	  -- Si l'execuci del procediment provoca una excepci es registra a la taula LOGPROCEDIMENT seguit
    -- del text "ERROR: " + detall de l'error.
       when others then
       v_idproc := seq_log.nextval;
	   pkg_log.reg_log(v_idproc, 'ERROR', substr(sqlerrm,1,500));
     dbms_output.put_line('No s''ha trobat el parmetre d''entrada '||p_any);

 end c3_despesa_total_any;


  --------------------------------
  --C4_DESPESES_CATEGORIA_CAMBRA--
  --------------------------------
 --procediment per fer un llistat de les despeses d'una cambra i any indicats pels
 --parmetres d'entrada desglossades segons la categoria de les despeses
 procedure c4_despeses_categoria_cambra(
        p_nomcambra varchar2,
        p_any       number
        ) as

      --variable per guardar el valor d'un identificador nic amb que es
      --registrar el procediment a la taula de LOGPROCEDIMENTS
      v_idproc number;

      --cursor que ens retornar totes les despeses asociades a la cambra i
      --l'any indicats en els parmetres agrupades per categoria .
      cursor c_descam is
       select distinct d.idcategoria, cat.descripciocategoria, sum (d.import) as despesa_categoria
          from despesa d, categoriadespesa cat
          where d.anydespesa = p_any
          and d.nomcambra = p_nomcambra
          and d.idcategoria = cat.idcategoria
          group by d.idcategoria, cat.descripciocategoria
          order by d.idcategoria;

      --variable on es guarden els valors obtinguts abans
      i c_descam%rowtype;
 begin

      --Instruccions per registrar el procediment a la taula de LOGPROCEDIMENTS
      v_idproc := seq_log.nextval;
      pkg_log.reg_log(v_idproc,'C4_DESPESES_CATEGORIA_CAMBRA - PARAMETRES: '||p_nomcambra||', '||p_any||'', 'LLISTAT DE SORTIDA PER STDOUT');

      -- Augmentem el tamany del buffer de l'output suficient perqu cpiga tot el missatge.
      dbms_output.enable(1000000);



      dbms_output.put_line('--------------------------------------------------');
      dbms_output.put_line('LLISTAT DE DESPESES DE LA CAMBRA: '||p_nomcambra);
      dbms_output.put_line('EN L''ANY: '||p_any);
      dbms_output.put_line('--------------------------------------------------');
      dbms_output.new_line;

      -- S'obre el cursor amb les dades de les despeses del parlamentari indicat
      open c_descam;
        fetch c_descam into i;

        if c_descam%found then
        close c_descam;
        for i in c_descam loop
          -- Si el cursor retorna registres s'escriuen al buffer de l'output.
         	dbms_output.put_line('IDENTIFICADOR DE CATEGORIA:    '||i.idcategoria);
          dbms_output.put_line('DESCRIPCIO DE LA CATEGORIA:    '||i.descripciocategoria);
          dbms_output.put_line('DESPESA TOTAL DE LA CATEGORIA: '||to_char(i.despesa_categoria));
          dbms_output.put_line('---');
          dbms_output.new_line;
        end loop;
      else
        dbms_output.put_line('--------------------------------------------------------');
        dbms_output.put_line('LA CONSULTA NO HA TROBAT CAP LA CAMBRA O L''ANY INDICATS');
        dbms_output.put_line('--------------------------------------------------------');
  		  dbms_output.new_line;
		  end if;

    --tractament d'excepcions
    exception
	  -- Si l'execuci del procediment provoca una excepci es registra a la taula LOGPROCEDIMENT seguit
    -- del text "ERROR: " + detall de l'error.
       when others then
       v_idproc := seq_log.nextval;
	   pkg_log.reg_log(v_idproc, 'ERROR', substr(sqlerrm,1,500));
     dbms_output.put_line('No s''ha trobat algun dels parmetres d''entrada: ' ||p_nomcambra|| 'o ' ||p_any);

 end c4_despeses_categoria_cambra;


  -------------------------
  --C5_PARL_SUPEREN_MITJA--
  -------------------------
 --procediment per fer un llistat dels parlamentaris de la cambra indicada que han
 --superat la mitjana de les despeses de tots els parlamentaris
 procedure c5_parl_superen_mitja(
        p_nomcambra varchar2,
        p_any       number
        ) as

      --variable per guardar el valor d'un identificador nic amb que es
      --registrar el procediment a la taula de LOGPROCEDIMENTS
      v_idproc number;

      --variables on emmagatzemarem els diferents valors calculats que ens
      --demanen
      sum_des_parl    number;
      avg_des_parl    number;
      avg_tots_parl   number;
      percentatge     number(8,2);
      trobat boolean;
      
    --cursor per saber si l'any existeix a la bbdd
      cursor c_any is
        select distinct anydespesa
        from despesa;
        
     --variable on guardar l'any
     anydes c_any%rowtype;
     

      --cursor que ens selecciona els diferents parlamentaris de la cambra
      --d'inters que ve donada pel parmetre d'entrada
      cursor c_parl is
          select distinct numparlamentari, nomcognoms, partitpolitic
          from parlamentari
          where nomcambra = p_nomcambra;

      --variable on guardar els resultats del cursor anterior
      i c_parl%rowtype;

      begin

      --Instruccions per registrar el procediment a la taula de LOGPROCEDIMENTS
      v_idproc := seq_log.nextval;
      pkg_log.reg_log(v_idproc,'C5_PARL_SUPEREN_MITJA - PARAMETRE: '||p_nomcambra||', '||p_any||'', 'LLISTAT DE SORTIDA PER STDOUT');

      -- Augmentem el tamany del buffer de l'output suficient perqu cpiga tot el missatge.
      dbms_output.enable(1000000);

      --Capalera output
      dbms_output.put_line('--------------------------------------------');
      dbms_output.put_line('LLISTAT DE PARLAMENTARIS');
      dbms_output.put_line('DE LA CAMBRA '||p_nomcambra);
      dbms_output.put_line('QUE SUPEREN LA MITJA DE DESPESES');
      dbms_output.put_line('EN L''ANY: '||p_any);
      dbms_output.put_line('--------------------------------------------');
      dbms_output.new_line;

      --inicialitzem les variables
      sum_des_parl   := '0';
      avg_des_parl   := '0';
      avg_tots_parl  := '0';
      percentatge    := '0';
      trobat := false;
      
       -- S'obre el cursor amb les dades dels anys
     open c_any;
        fetch c_any into anydes;

        if c_any%found then
        close c_any;
        
        --mirem si l'any indicat es troba a la bd
        for anydes in c_any loop
          if anydes.anydespesa = p_any then
          trobat := true;
          end if;
          end loop;
          if trobat = false then 
            dbms_output.put_line('LA CONSULTA NO HA TROBAT L''ANY INDICAT');
          
     else
      
      --calcul de la despesa mitja del parlamentaris de la cambra indicada
      --en els parmetres
      select avg(d.import) into avg_tots_parl
        from despesa d, parlamentari pl
                where pl.nomcambra = p_nomcambra
                and pl.numparlamentari = d.numparlamentari
                and d.anydespesa = p_any;
                if avg_tots_parl is null
                then avg_tots_parl := '0';
                end if;

      -- S'obre el cursor amb les dades dels parlamentaris per calcular la mitja
      -- de la despesa de cadascun i la suma total
      open c_parl;
        fetch c_parl into i;

        if c_parl%found then
        close c_parl;

        --fem els clculs per cada parlamentari trobat i guardat a c_parl
      for i in c_parl loop
          select avg(d.import) into avg_des_parl
                from despesa d
                where i.numparlamentari = d.numparlamentari
                and d.anydespesa = p_any;
                if avg_des_parl is null
                then avg_des_parl := '0';
                end if;
          select sum(d.import) into sum_des_parl
                from despesa d
                where i.numparlamentari = d.numparlamentari
                and d.anydespesa = p_any;
                if sum_des_parl is null
                then sum_des_parl := '0';
                end if;

         if avg_des_parl > avg_tots_parl then
                percentatge := (avg_des_parl - avg_tots_parl)*100 / avg_tots_parl;

           -- Presentem els resultats per l'stdout.
         	dbms_output.put_line('NOM I COGNOMS:                    '||i.nomcognoms);
          dbms_output.put_line('PARTIT POLITIC:                   '||i.partitpolitic);
          dbms_output.put_line('DESPESA DIRECTA ASSOCIADA:        '||to_char(sum_des_parl));
          dbms_output.put_line('PERCENTATGE PER SOBRE LA MITJA:   '||to_char(percentatge));
          dbms_output.put_line('---');
          dbms_output.new_line;

         else

            dbms_output.put_line('El parlamentari ' ||i.nomcognoms );
            dbms_output.put_line('no supera la mitja de despeses');
            dbms_output.put_line('----');
            dbms_output.new_line;
        end if; 
         end loop;
     else
        dbms_output.put_line('-----------------------------------------------------');
        dbms_output.put_line('LA CONSULTA NO HA TROBAT LA CAMBRA INDICADA');
        dbms_output.put_line('------------------------------------------------------');
  		  dbms_output.new_line;
		  end if;
    end if;
  end if;
  
     --tractament d'excepcions
     exception
	  -- Si l'execuci del procediment provoca una excepci es registra a la taula LOGPROCEDIMENT seguit
    -- del text "ERROR: " + detall de l'error.
       when others then
       v_idproc := seq_log.nextval;
	   pkg_log.reg_log(v_idproc, 'ERROR', substr(sqlerrm,1,500));
     dbms_output.put_line('No s''ha trobat algun dels parmetres d''entrada: ' ||p_nomcambra|| 'o ' ||p_any);

 end c5_parl_superen_mitja;


  ----------------------
  --C6_ESTAT_COMPTABLE--
  ----------------------
--procediment per calcular l'estat comptable de totes les cambres en l'any
 --indicat pel parmetre d'entrada
procedure c6_estat_comptable(
      p_any number
      ) as

      --variable per guardar el valor d'un identificador nic amb que es
      --registrar el procediment a la taula de LOGPROCEDIMENTS
      v_idproc number;

      --variables on emmagatzemarem els diferents valors calculats que ens
      --demanen
      pap  number;
      ap   number;
      pab  number;
      ab   number;
      trobat boolean;
      
      --cursor per saber si l'any existeix a la bbdd
      cursor c_any is
        select distinct anydespesa
        from despesa;
        
     --variable on guardar l'any
     anydes c_any%rowtype;

       --cursor que ens selecciona els noms de les diferents cambres
      cursor c_nomcam is
          select distinct nomcambra, numparlamentaris
          from cambra;

      --variable on guardar els resultats del cursor anterior
      i c_nomcam%rowtype;


      begin

      --Instruccions per registrar el procediment a la taula de LOGPROCEDIMENTS
      v_idproc := seq_log.nextval;
      pkg_log.reg_log(v_idproc,'C6_ESTAT_COMPTABLE - PARAMETRE: '||p_any||'', 'LLISTAT DE SORTIDA PER STDOUT');

      -- Augmentem el tamany del buffer de l'output suficient perqu cpiga tot el missatge.
      dbms_output.enable(1000000);

     --inicilitzem les variables on guardarem els import de cada estat comptable
      pap  := '0';
      ap   := '0';
      pab  := '0';
      ab   := '0';
      trobat := false;
      
       -- S'obre el cursor amb les dades dels anys
     open c_any;
        fetch c_any into anydes;

        if c_any%found then
        close c_any;
        
        --mirem si l'any indicat es troba a la bd
        for anydes in c_any loop
          if anydes.anydespesa = p_any then
          trobat := true;
          end if;
          end loop;
      end if;
     
     if trobat = false then 
            dbms_output.put_line('LA CONSULTA NO HA TROBAT L''ANY INDICAT');
          
     else
    -- S'obre el cursor amb les dades de les despeses del parlamentari indicat
     open c_nomcam;
        fetch c_nomcam into i;

        if c_nomcam%found then
        close c_nomcam;

      --fem els clculs necessaris sobre cada cambra obtinguda en la selecci anterior
      for i in c_nomcam loop
          select sum(d.import) into pap
                from despesa d
                where i.nomcambra = d.nomcambra
                and d.idestatdespesa = 'PAP'
                and d.anydespesa = p_any;
                if pap is null
                then pap := '0';
                end if;
          select sum(d.import) into ap
                from despesa d
                where i.nomcambra = d.nomcambra
                and d.idestatdespesa = 'AP'
                and d.anydespesa = p_any;
                if ap is null
                then ap := '0';
                end if;
           select sum(d.import) into pab
                from despesa d
                where i.nomcambra = d.nomcambra
                and d.idestatdespesa = 'PAB'
                and d.anydespesa = p_any;
                if pab is null
                then pab := '0';
                end if;
           select sum(d.import) into ab
                from despesa d
                where i.nomcambra = d.nomcambra
                and d.idestatdespesa = 'AB'
                and d.anydespesa = p_any;
                if ab is null
                then ab := '0';
                end if;

           -- Presentem els resultats per l'stdout.
         	dbms_output.put_line('NOM CAMBRA:                     '||i.nomcambra);
          dbms_output.put_line('NOMBRE DE PARLAMENARIS:         '||i.numparlamentaris);
          dbms_output.put_line('DESPESES PENDENTS D''APROVACI:  '||to_char(pap));
          dbms_output.put_line('DESPESES APROVADES:             '||to_char(ap));
          dbms_output.put_line('DESPESA PENDENT D''ABONAMENT:    '||to_char(pab));
          dbms_output.put_line('DESPESA ABONADES                '||to_char(ab));
          dbms_output.put_line('---');
          dbms_output.new_line;
        end loop;
     
       end if;
     
     end if;

    
     --tractrament d'excepcions
     exception
	  -- Si l'execuci del procediment provoca una excepci es registra a la taula LOGPROCEDIMENT seguit
    -- del text "ERROR: " + detall de l'error.
       when others then
       v_idproc := seq_log.nextval;
	   pkg_log.reg_log(v_idproc, 'ERROR', substr(sqlerrm,1,500));
     dbms_output.put_line('No s''ha trobat el parmetre d''entrada ' ||p_any);

end c6_estat_comptable;


end pkg_consultes;

/

spool off;